#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <net/if.h>
#include <uci.h>
#include <string.h>

#include <netlink/netlink.h>
#include <netlink/route/rtnl.h>
#include <netlink/socket.h>
#include <netlink/msg.h>

#include <sys/ioctl.h>
#include <net/if.h>

#include <libubox/blobmsg.h>
#include <libubox/blobmsg_json.h>
#include <libubox/uloop.h>
#include <libubox/ustream.h>
#include <libubox/utils.h>
#include <libubus.h>

#include <easy/easy.h>

#include <map2.h>
#include <i1905_wsc.h>
#include "config.h"
#include "agent.h"


int if_updown(const char *ifname, bool up)
{
	int fd;
	struct ifreq ifr;
	short flags;

	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		return -1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, ifname, IFNAMSIZ);

	if (ioctl(fd, SIOCGIFFLAGS, &ifr) != 0) {
		close(fd);
		return -1;
	}

	flags = ifr.ifr_flags;

	if (up && !(flags & IFF_UP))
		ifr.ifr_flags |= IFF_UP;

	if (!up && (flags & IFF_UP))
		ifr.ifr_flags &= ~IFF_UP;

	if ((flags != ifr.ifr_flags) && (0 != ioctl(fd, SIOCSIFFLAGS, &ifr))) {
		close(fd);
		return -1;
	}

	close(fd);
	return 0;
}

static int func(struct nl_msg *msg, void *arg)
{
	struct nlmsghdr *nlh = nlmsg_hdr(msg);

	switch (nlh->nlmsg_type) {
	case RTM_GETLINK:
	case RTM_NEWLINK:
	//case RTM_DELLINK:
		{
			struct ifinfomsg *ifi;
			char ifname[16] = {0};
			struct agent *a = (struct agent *) arg;
			char *bridge = a->cfg.al_bridge;
			int ret;

			ifi = NLMSG_DATA(nlh);

			if_indextoname(ifi->ifi_index, ifname);

			//printf("ifname = %s  ifindex = %d  status = %s  action = %s\n",
			//       ifname,
			//       ifi->ifi_index,
			//       !!(ifi->ifi_flags & IFF_UP) ? "up" : "down",
			//       nlh->nlmsg_type == RTM_GETLINK ? "RTM_GETLINK" :
			//       nlh->nlmsg_type == RTM_NEWLINK ? "RTM_NEWLINK" :
			//       "RTM_DELLINK");

			//if (!(ifi->ifi_flags & IFF_UP))
			//	break;

			if (!strstr(ifname, "wds"))
				break;

#if 0			// Not needed for now, maybe never?
			// TODO: find a better place to do this
			if (a->ts.active && !strstr(ifname, "vlan")) {
				char buf[32] = { 0 };
				snprintf(buf, 32, "%s_vlan", ifname);
				if (if_isbridge_interface(buf) == -1){
					// create tagged wds
					runCmd("/lib/wifi/multiap ts create bh %s %d 2",
					       ifname, a->cfg.pcfg->pvid );
				}
				break;
			}
#endif

			if (if_isbridge_interface(ifname)) {
				int i, num, max = 32;
				char if_list[max][16];
				bool found = false;

				if (br_get_iflist(bridge, &num, if_list)) {
					printf("Failed to check bridge affiliation\n");
					break;
				}
				max = (num < max) ? num : max;
				for (i = 0; i < max; i++) {
					if (!strncmp(ifname, if_list[i], 15)) {
						printf("Interface %s is already a part of bridge\n",
						       ifname);
						found = true;
						break;
					}
				}
				if (found)
					break;
			}

			printf("Adding interface %s to bridge %s!\n", ifname,
			       bridge);

			/* add wds iface to bridge */
			ret = br_addif(bridge, ifname);
			if (!ret)
				printf("Successfully added interface %s to bridge %s\n",
				       ifname, bridge);

			/* bring up wds interface */
			//ret = if_setflags(ifname, IFF_UP);
			ret = if_updown(ifname, true);
			if (!ret)
				printf("Successfully brought up interface %s\n",
						ifname);
			break;
		}
	default:
		break;

	}

	return 0;
}

int nl_loop(void *arg)
{
	struct nl_sock *sk;

	sk = nl_socket_alloc();
	if (!sk)
		return 1;

	nl_socket_disable_seq_check(sk);

	nl_socket_modify_cb(sk, NL_CB_VALID, NL_CB_CUSTOM, func, arg);

	nl_connect(sk, NETLINK_ROUTE);
	nl_socket_add_memberships(sk, RTNLGRP_LINK, 0);
	nl_rtgen_request(sk, RTM_GETLINK, AF_UNSPEC, NLM_F_DUMP);

	for (;;)
		nl_recvmsgs_default(sk);

	return 0;
}
